@@ -82,7 +82,7 @@ class Node(EventTarget):
         self.namespaceURI = "http://www.w3.org/1999/xhtml"
         # self.nodePrincipal = None
         self.outerText = None
-        self.ownerDocument = None
+        # self.ownerDocument = None
         self.parentNode = None
         self.prefix = None  # 🗑️
         self._update_parents()
@@ -111,6 +111,23 @@ class Node(EventTarget):
         except Exception as e:
             print('unable to update parent', e)
 
+
+    def _iterate(self, element, callback):
+        '''
+        private.
+        currently used for querySelector
+        '''
+        from domonic.javascript import Array
+        nodes = Array()
+        nodes.push(element)
+        while len(nodes) > 0:
+            # print('checking')
+            element = nodes.shift()
+            callback(element)
+            nodes.unshift(*element.children)
+
+
+
     @property
     def rootNode(self):
         """[read-only property returns a Node object representing the topmost node in the tree,
@@ -304,6 +321,26 @@ class Node(EventTarget):
     # setUserData() 🗑️
 
 
+class TextNode(Node):
+    """ not tested """
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    @property
+    def textContent(self):
+        """ Sets or returns the textual content of a node and its descendants """
+        # return f" {' '*len(self.name)}{' '*len(self.attributes)} {self.content}  {' '*len(self.name)} "
+        return self.nodeValue
+
+    @textContent.setter
+    def textContent(self, content):
+        """ Sets or returns the textual content of a node and its descendants """
+        # if type(content) is not str:
+        # raise ValueError()
+        self.nodeValue = content
+
+
 class ParentNode(object):
     """ not tested yet """
 
@@ -405,8 +442,51 @@ class Element(Node):
         self.tagName
         self.style = None  # Style(self)  # = #'test'#Style()
         self.shadowRoot = None
+        self.dir = None
         super().__init__(*args, **kwargs)
 
+    def _getElementById(self, _id):
+        # TODO - i think i need to build a hash map of IDs to positions on the tree
+        # for now I'm going using recursion so this is a bit of a hack to do a few levels
+        if self.getAttribute('id') == _id:
+            return self
+        try:
+            for child in self.childNodes:
+                match = child._getElementById(_id)
+                if match:
+                    return match
+        except Exception as e:
+            # print('fail', e)
+            pass # TODO - dont iterate strings
+        return False
+
+    def _matchElement(self, element, query):
+        """
+        tries to match an element based on the query
+        at moment very basic. i.e. single level. just checks between id/tag/class
+        """
+        try:
+            if query[0] == '#':
+                if element.getAttribute('id') == query.split('#')[1]:
+                    return True
+        except Exception as e:
+            pass
+
+        try:
+            if element.tagName.lower() == query.lower():
+                return True
+        except Exception as e:
+            pass
+
+        try:
+            if query[0] == '.':
+                if query.split('.')[1] in element.classList:
+                    return True
+        except Exception as e:
+            pass
+
+        return False
+
     # elem.attachShadow({mode: open|closed})
     def attachShadow(self, obj):
         self.shadowRoot = ShadowRoot(self, obj['mode'])
@@ -500,9 +580,15 @@ class Element(Node):
         """ Sets or returns whether the content of an element is editable or not """
         raise NotImplementedError
 
+    @property
     def dir(self):
-        """ Sets or returns the value of the dir attribute of an element """
-        raise NotImplementedError
+        """ returns the value of the dir attribute of an element """
+        return self.getAttribute('dir')
+
+    @dir.setter
+    def dir(self, direction: str = 'auto'):
+        """ Sets the value of the dir attribute of an element """
+        self.setAttribute('dir', direction)
 
     def exitFullscreen(self):
         """ Cancels an element in fullscreen mode """
@@ -545,9 +631,9 @@ class Element(Node):
         '''Returns the size of an element and its position relative to the viewport'''
         raise NotImplementedError
 
-    def getElementsByClassName(self):
+    def getElementsByClassName(self, className):
         '''Returns a collection of all child elements with the specified class name'''
-        raise NotImplementedError
+        return self.querySelectorAll('.' + className)
 
     def getElementsByTagName(self, tag: str) -> List:
         """ Returns a collection of all child elements with the specified tag name """
@@ -650,7 +736,19 @@ class Element(Node):
 
     def normalize(self):
         '''Joins adjacent text nodes and removes empty text nodes in an element'''
-        raise NotImplementedError
+        content = []
+        for s in self.args:
+            if type(s) == TextNode:
+                content.append(s.textContent)
+                continue
+            if type(s) != str:
+                content.append(str(s))
+                continue
+            content.append(s)
+
+        self.args = content
+        self.args = [TextNode(' '.join([str(s) for s in self.args]))]
+        return self.args
 
     def offsetHeight(self):
         ''' Returns the height of an element, including padding, border and scrollbar'''
@@ -672,6 +770,7 @@ class Element(Node):
         ''' Returns the vertical offset position of an element'''
         raise NotImplementedError
 
+    @property
     def ownerDocument(self):
         """ Returns the root element (document object) for an element """
         return self.rootNode
@@ -700,17 +799,29 @@ class Element(Node):
                         return self.parentNode.args[count - 1]
         return None
 
-    def querySelector(self):
+    def querySelector(self, query):
         '''Returns the first child element that matches a specified CSS selector(s) of an element'''
-        raise NotImplementedError
+        try:
+            return self.querySelectorAll(query)[0]
+        except Exception as e:
+            return None
 
-    def querySelectorAll(self):
+    def querySelectorAll(self, query):
         """ Returns all child elements that matches a specified CSS selector(s) of an element """
-        raise NotImplementedError
+        elements = []
+
+        def anon(el):
+            if self._matchElement(el, query):
+                elements.append(el)
+
+        self._iterate(self, anon)
+        return elements
 
     def remove(self):
         """ Removes the element from the DOM """
-        raise NotImplementedError
+        # self.ownerDocument
+        # raise NotImplementedError
+        self.parentNode.removeChild(self)
 
     def removeAttribute(self, attribute: str):
         """ Removes a specified attribute from an element """
@@ -967,9 +1078,10 @@ class Document(Element):
             return Event()
         return Event()
 
-    # def createTextNode():
+    @staticmethod
+    def createTextNode(text):
         '''Creates a Text node'''
-        # return
+        return TextNode(text)
 
     # def defaultView(self):
         ''' Returns the window object associated with a document, or null if none is available.'''
@@ -1024,13 +1136,24 @@ class Document(Element):
         # '''Returns a Boolean value indicating whether the document can be viewed in fullscreen mode'''
         # return
 
-    def getElementById(self):
+    def getElementById(self, _id):
         '''Returns the element that has the ID attribute with the specified value'''
-        raise NotImplementedError
+        for each in self.childNodes:
+            if each.getAttribute('id') == _id:
+                return each
+            try:
+                for child in each.childNodes:
+                    match = child._getElementById(_id)
+                    # TODO - i think i need to build a hash map of IDs to positions on the tree
+                    # for now I'm going to use recursion and add this same method to Element
+                    if match:
+                        return match
 
-    def getElementsByClassName(self):
-        '''Returns a NodeList containing all elements with the specified class name'''
-        raise NotImplementedError
+            except Exception as e:
+                # print('doh', e)
+                pass  # TODO - dont iterate strings
+
+        return False
 
     def getElementsByName(self):
         '''Returns a NodeList containing all elements with a specified name'''
@@ -1078,32 +1201,20 @@ class Document(Element):
         tags = re.findall(pattern, str(self))
         return tags
 
-    def normalize(self):
-        '''Removes empty Text nodes, and joins adjacent nodes'''
-        raise NotImplementedError
-
     def normalizeDocument(self):
         '''Removes empty Text nodes, and joins adjacent nodes'''
         raise NotImplementedError
 
     # def open(self):
-        '''Opens an HTML output stream to collect output from document.write()'''
+        # '''Opens an HTML output stream to collect output from document.write()'''
         # return
 
-    def querySelector(self):
-        '''Returns the first element that matches a specified CSS selector(s) in the document'''
-        raise NotImplementedError
-
-    def querySelectorAll(self):
-        '''Returns a static NodeList containing all elements that matches a specified CSS selector(s) in the document'''
-        raise NotImplementedError
-
     # def readyState(self):
-        ''' Returns the (loading) status of the document'''
+        # ''' Returns the (loading) status of the document'''
         # return
 
     # def referrer():
-        ''' Returns the URL of the document that loaded the current document'''
+        # ''' Returns the URL of the document that loaded the current document'''
         # return
 
     def renameNode(self, node, namespaceURI, nodename):
